#include "account.h"
#include "chat.h"
#include "context.h"
#include "event.h"
#include "eventemitter.h"
#include <QDebug>
#include <QStandardPaths>

namespace dc {

std::shared_ptr<Account> Account::inst = nullptr;

Account::Account(QString path)
{
    const int writeable = true;
    m_accounts = dc_accounts_new(path.toUtf8().constData(), writeable);
    // install emiter
    m_emit = new EventEmitter(dc_accounts_get_event_emitter(m_accounts));
    qDebug() << "Seting path on " << path;
}

Account::~Account()
{
    stopIo();
    dc_accounts_unref(m_accounts);
    m_emit->wait();
    delete m_emit;
}

void Account::init(QString datapath)
{
    if (datapath.isEmpty()) {
        auto path = QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation);
        inst = std::shared_ptr<Account>(new Account(path));
    } else
        inst = std::shared_ptr<Account>(new Account(datapath));
    qRegisterMetaType<size_t>("size_t");
    qRegisterMetaType<uint32_t>("uint32_t");
}

void Account::close()
{
    inst = nullptr;
}

std::shared_ptr<Account> Account::getInstance()
{
    return inst;
}

void Account::startIo()
{
    dc_accounts_start_io(m_accounts);
}
void Account::stopIo()
{
    dc_accounts_stop_io(m_accounts);
}

void Account::maybeNetWork()
{
    dc_accounts_maybe_network(m_accounts);
}

void Account::maybeNetWorkLost()
{
    dc_accounts_maybe_network_lost(m_accounts);
}

void Account::backgroundFetch(uint64_t timeout)
{
    dc_accounts_background_fetch(m_accounts, timeout);
}

std::shared_ptr<Context> Account::getAccount(uint32_t id)
{
    dc_context_t* ctx = dc_accounts_get_account(m_accounts, id);
    if (ctx != nullptr) {
        auto acc = std::shared_ptr<Context>(new Context(ctx));
        return acc;
    } else
        return nullptr;
}

std::shared_ptr<Context> Account::getSelectedAccount()
{
    dc_context_t* ctx = dc_accounts_get_selected_account(m_accounts);
    if (ctx != nullptr) {
        auto acc = std::shared_ptr<Context>(new Context(ctx));
        return acc;
    } else
        return nullptr;
}

void Account::setSelectAccount(uint32_t id)
{
    dc_accounts_select_account(m_accounts, id);
}

void Account::removeAccount(uint32_t id)
{
    dc_accounts_remove_account(m_accounts, id);
}

uint32_t Account::addAcount()
{
    return dc_accounts_add_account(m_accounts);
}

QList<uint32_t> Account::getAll()
{
    dc_array_t* arr = dc_accounts_get_all(m_accounts);
    QList<uint32_t> id;
    for (int i = 0; i < dc_array_get_cnt(arr); i++) {
        id.append(dc_array_get_id(arr, i));
    }
    dc_array_unref(arr);
    return id;
}

void Account::startEventLoop()
{
    // start io
    startIo();
    connect(m_emit, &EventEmitter::gotEvent, this, &Account::processEvent, Qt::DirectConnection);
    m_emit->start();
}

std::shared_ptr<Context> Account::getUser(const QString& addr)
{
    auto list = getAll();
    for (auto id : list) {
        auto ctx = getAccount(id);
        if (ctx->getConfig("addr") == addr) {
            return ctx;
        }
    }
    return nullptr;
}

void Account::processEvent(QSharedPointer<Event> event)
{
    int acc_id;
    std::shared_ptr<Context> ctx;
    std::shared_ptr<Chat> chat;
    int stats;
    switch (event->getId()) {
        /*
    case DC_EVENT_SMTP_CONNECTED:
        // qInfo("SMTP connected: %s", qUtf8Printable(event->getData2Str()));
        break;
    case DC_EVENT_IMAP_CONNECTED:
        // qInfo("IMAP connected: %s", qUtf8Printable(event->getData2Str()));
        break;
    case DC_EVENT_SMTP_MESSAGE_SENT:
        // qInfo("SMTP message sent: %s", qUtf8Printable(event->getData2Str()));
        break;
    case DC_EVENT_IMAP_MESSAGE_DELETED:
        // qInfo("IMAP message deleted: %s", qUtf8Printable(event->getData2Str()));
        break;
    case DC_EVENT_IMAP_MESSAGE_MOVED:
        // qInfo("IMAP message moved: %s", qUtf8Printable(event->getData2Str()));
        break;
    case DC_EVENT_NEW_BLOB_FILE:
        // qInfo("New blob file: %s", qUtf8Printable(event->getData2Str()));
        break;
    case DC_EVENT_DELETED_BLOB_FILE:
        // qInfo("Deleted blob file: %s", qUtf8Printable(event->getData2Str()));
        break;
    case DC_EVENT_ERROR_SELF_NOT_IN_GROUP:
        // qCritical("Self not in group error: %s", qUtf8Printable(event->getData2Str()));
        break;

    case DC_EVENT_INFO:
        // qInfo("%s", qUtf8Printable(event->getData2Str()));
        break;
    case DC_EVENT_WARNING:
        // qWarning("%s", qUtf8Printable(event->getData2Str()));
        break;
    case DC_EVENT_ERROR:
        // qCritical("%s", qUtf8Printable(event->getData2Str()));
        break;
    case DC_EVENT_IMAP_INBOX_IDLE:
        break;

    case DC_EVENT_CONTACTS_CHANGED:
        qInfo("Contact change");
        break;
    case DC_EVENT_CHATLIST_CHANGED:
        qInfo("Chatlist change");
        break;
    case DC_EVENT_INCOMING_MSG_BUNCH:
        qInfo("Msg brunch");
        break;

    case DC_EVENT_MSG_READ:
        break;
        //*/

    case DC_EVENT_MSG_FAILED:
        emit messageFalse(event->getAccountId(), event->getData1Int(), event->getData2Int());
        break;
    case DC_EVENT_MSG_DELIVERED:
        emit messagesDelevery(event->getAccountId(), event->getData1Int(), event->getData2Int());
        break;
    case DC_EVENT_CONNECTIVITY_CHANGED:
        ctx = dc::Account::getAccount(event->getAccountId());
        stats = ctx->getConnectivity();
        if (stats < 2000)
            stats = DC_CONNECTIVITY_NOT_CONNECTED;
        else if (stats < 3000)
            stats = DC_CONNECTIVITY_CONNECTING;
        else if (stats < 4000)
            stats = DC_CONNECTIVITY_WORKING;
        else {
            stats = DC_CONNECTIVITY_CONNECTED;
        }
        emit connectivityChange(event->getAccountId(), stats);
        break;
    case DC_EVENT_CHATLIST_ITEM_CHANGED:
        emit chatListItemChange(event->getAccountId(), event->getData1Int());
        break;
    case DC_EVENT_CHAT_DELETED:
        emit chatDeleted(event->getAccountId(), event->getData1Int());
        break;
    case DC_EVENT_CHAT_MODIFIED:
        emit chatModified(event->getAccountId(), event->getData1Int());
        break;
    case DC_EVENT_MSGS_CHANGED:
        emit messagesChanged(event->getAccountId(), event->getData1Int(), event->getData2Int());
        break;
    case DC_EVENT_INCOMING_MSG:
        ctx = Account::getInstance()->getAccount(event->getAccountId());
        chat = ctx->getChat(event->getData1Int());
        if (chat->isContactRequest()) {
            qDebug() << "auto accepp chat";
            ctx->acceptChat(event->getData1Int());
        }
        emit incomingMessage(event->getAccountId(), event->getData1Int(), event->getData2Int());
        break;
    case DC_EVENT_MSGS_NOTICED:
        emit messagesNoticed(event->getAccountId(), event->getData1Int());
        break;
    case DC_EVENT_CONFIGURE_PROGRESS:
        emit configureProgress(event->getAccountId(), event->getData1Int(), event->getData2Str());
        acc_id = event->getAccountId();
        if (event->getData1Int() == 1000) {
            ctx = Account::getInstance()->getAccount(acc_id);
            ctx->startIo();
        }
        break;
    case DC_EVENT_IMEX_PROGRESS:
        emit imexProgress(event->getAccountId(), event->getData1Int());
        break;
    default:
        // qDebug() << "Not processing " << event->getId();
        break;
    }
}

} // namespace dc
