#include "context.h"
#include "account.h"
#include "chat.h"
#include "chatlist.h"
#include "contact.h"
#include "lot.h"
#include "message.h"
#include <QDebug>

namespace dc {
Context::Context(dc_context_t* mctx)
{
    m_context = mctx;
}

Context::~Context()
{
    dc_context_unref(m_context);
}

void Context::configure()
{
    dc_configure(m_context);
}

bool Context::isConfigured() const
{
    return dc_is_configured(m_context);
}

void Context::startIo()
{
    dc_start_io(m_context);
}

void Context::stopIo()
{
    dc_stop_io(m_context);
}

void Context::maybeNetwork()
{
    dc_maybe_network(m_context);
}

int Context::getConnectivity()
{
    return dc_get_connectivity(m_context);
}

QString Context::getInfo()
{
    char* info = dc_get_info(m_context);
    QString result { info };
    dc_str_unref(info);
    return result;
}

std::shared_ptr<Chatlist> Context::getChatlist(int flags, QString query)
{
    QByteArray utf8Query = query.toUtf8();
    dc_chatlist_t* chatlist = dc_get_chatlist(m_context,
        flags,
        query.isEmpty() ? nullptr : utf8Query.constData(),
        0);
    auto p = new Chatlist(chatlist);
    return std::shared_ptr<Chatlist>(p);
}

uint32_t Context::createChatByContactId(uint32_t contactId)
{
    auto cid = dc_get_chat_id_by_contact_id(m_context, contactId);
    if (cid)
        return cid;
    else {
        cid = dc_create_chat_by_contact_id(m_context, contactId);
    }
}

uint32_t Context::createGroupChat(const QString& name, bool secure)
{
    QByteArray str = name.toUtf8();
    return dc_create_group_chat(m_context, secure, str.constData());
}

int Context::removeContactFromChat(uint32_t chat_id, uint32_t contact_id)
{
    auto ret = dc_remove_contact_from_chat(m_context, chat_id, contact_id);
    auto acc = dc::Account::getInstance();
    emit acc->removeContactFromChat(getId(), chat_id, contact_id);
    return ret;
}

int Context::addContactToChat(uint32_t chat_id, uint32_t contact_id)
{
    return dc_add_contact_to_chat(m_context, chat_id, contact_id);
}

QList<uint32_t> Context::getChatContacts(uint32_t chat_id)
{
    dc_array_t* arr = dc_get_chat_contacts(m_context, chat_id);
    auto size = dc_array_get_cnt(arr);
    QList<uint32_t> res;
    // qDebug() << QString::number(chat_id) << " get contact :size" << QString::number(size);
    for (int i = 0; i < size; i++) {
        res.append(dc_array_get_id(arr, i));
    }
    dc_array_unref(arr);
    return res;
}

std::shared_ptr<Lot> Context::chatlistGetSummary2(uint32_t chat_id,
    uint32_t msg_id)
{
    auto lt = dc_chatlist_get_summary2(m_context, chat_id, msg_id);
    return std::shared_ptr<Lot>(new Lot(lt));
}

uint32_t Context::getId()
{
    return dc_get_id(m_context);
}

void Context::setChatVisibility(uint32_t chatId, int visibility)
{
    dc_set_chat_visibility(m_context, chatId, visibility);
}

void Context::deleteChat(uint32_t chatId)
{
    dc_delete_chat(m_context, chatId);
}

void Context::blockChat(uint32_t chatId)
{
    dc_block_chat(m_context, chatId);
}

void Context::acceptChat(uint32_t chatId)
{
    dc_accept_chat(m_context, chatId);
}

QString Context::getChatEncrinfo(uint32_t chatId)
{
    char* encrinfo = dc_get_chat_encrinfo(m_context, chatId);
    QString result { encrinfo };
    dc_str_unref(encrinfo);
    return result;
}

uint32_t Context::getChatEphemeralTimer(uint32_t chatId)
{
    return dc_get_chat_ephemeral_timer(m_context, chatId);
}

std::shared_ptr<Chat> Context::getChat(uint32_t chatId)
{
    dc_chat_t* chat = dc_get_chat(m_context, chatId);
    return std::shared_ptr<Chat>(new Chat { chat });
}

QList<int> Context::getMsgIdList(uint32_t chatId, int flag)
{
    QList<int> result;
    dc_array_t* msgIdArray = dc_get_chat_msgs(m_context, chatId, flag, 0);
    for (size_t i = 0; i < dc_array_get_cnt(msgIdArray); i++) {
        result << dc_array_get_id(msgIdArray, i);
    }
    dc_array_unref(msgIdArray);
    return result;
}

QList<int> Context::getContacts(uint32_t flags, QString query)
{
    QList<int> result;
    QByteArray utf8Query = query.toUtf8();
    dc_array_t* contactsArray = dc_get_contacts(m_context, flags, utf8Query.constData());
    for (size_t i = 0; i < dc_array_get_cnt(contactsArray); i++) {
        result << dc_array_get_id(contactsArray, i);
    }
    dc_array_unref(contactsArray);
    return result;
}

int Context::getFreshMsgCnt(uint32_t chatId)
{
    return dc_get_fresh_msg_cnt(m_context, chatId);
}

void Context::marknoticedChat(uint32_t chatId)
{
    return dc_marknoticed_chat(m_context, chatId);
}

void Context::markseenMsgs(const QVector<uint32_t>& msg_ids)
{
    dc_markseen_msgs(m_context, msg_ids.constData(), msg_ids.count());
}

std::shared_ptr<Message> Context::getMessage(uint32_t msgId)
{
    dc_msg_t* message = dc_get_msg(m_context, msgId);
    auto m = new Message(message);
    return std::shared_ptr<Message>(m);
}

std::shared_ptr<Contact> Context::getContact(uint32_t contactId)
{
    dc_contact_t* contact = dc_get_contact(m_context, contactId);
    return std::shared_ptr<Contact>(new Contact { contact });
}

/**
 * delete all message in list
 */
void Context::deleteMsgs(const QVector<uint32_t>& msg)
{
    dc_delete_msgs(m_context, msg.constData(), msg.count());
}

uint32_t Context::createContact(QString name, QString addr)
{
    QByteArray utf8name = name.toUtf8();
    QByteArray utf8addr = addr.toUtf8();
    return dc_create_contact(m_context, utf8name.constData(), utf8addr.constData());
}

int Context::deleteContact(uint32_t contact_id)
{
    return dc_delete_contact(m_context, contact_id);
}

QString Context::getBlobdir()
{
    char* blobdir = dc_get_blobdir(m_context);
    QString result { blobdir };
    dc_str_unref(blobdir);
    return result;
}

bool Context::setConfig(QString key, QString value)
{
    QByteArray utf8Key = key.toUtf8();
    QByteArray utf8Value = value.toUtf8();
    return dc_set_config(m_context, utf8Key.constData(), utf8Value.constData());
}

QString Context::getConfig(QString key)
{
    QByteArray utf8Key = key.toUtf8();
    char* value = dc_get_config(m_context, utf8Key.constData());
    QString result { value };
    dc_str_unref(value);
    return result;
}

QString Context::getMessageInfo(uint32_t msgId)
{
    char* info = dc_get_msg_info(m_context, msgId);
    QString result { info };
    dc_str_unref(info);
    return result;
}

QString Context::getMessageHtml(uint32_t msgId)
{
    char* html = dc_get_msg_html(m_context, msgId);
    QString result { html };
    dc_str_unref(html);
    return result;
}

uint32_t Context::sendTextMessage(uint32_t chatId, QString textToSend)
{
    QByteArray utf8Text = textToSend.toUtf8();
    auto id = dc_send_text_msg(m_context, chatId, utf8Text.constData());
    emit dc::Account::getInstance()->outGoingMessage(getId(), chatId, id);
    return id;
}

void Context::setDraft(uint32_t chatId, std::shared_ptr<Message>& message)
{
    dc_set_draft(m_context, chatId, message ? message->m_message : nullptr);
}

std::shared_ptr<Message> Context::getDraft(uint32_t chatId)
{
    dc_msg_t* draft = dc_get_draft(m_context, chatId);
    if (draft) {
        return std::shared_ptr<Message>(new Message { draft });
    } else {
        return NULL;
    }
}

bool Context::setChatMuteDuration(uint32_t chatId, int64_t duration)
{
    return dc_set_chat_mute_duration(m_context, chatId, duration);
}

std::shared_ptr<Message> Context::newMessage(int viewtype)
{
    dc_msg_t* message = dc_msg_new(m_context, viewtype);
    return std::shared_ptr<Message>(new Message(message));
}

uint32_t Context::sendMessage(uint32_t chatId, std::shared_ptr<Message>& message)
{
    auto id = message->getId();
    emit dc::Account::getInstance()->outGoingMessage(getId(), chatId, id);
    return dc_send_msg(m_context, chatId, message->m_message);
}

void Context::importBackup(QString tarfile)
{
    QByteArray utf8Text = tarfile.toUtf8();
    return dc_imex(m_context, DC_IMEX_IMPORT_BACKUP, utf8Text.constData(), NULL);
}

std::shared_ptr<Lot> Context::checkQr(const QString& qr)
{
    QByteArray data = qr.toUtf8();
    auto ret = dc_check_qr(m_context, data.constData());
    return std::shared_ptr<Lot>(new Lot(ret));
}

QString Context::getSecurejoinQrSvg(uint32_t id)
{
    auto str = dc_get_securejoin_qr_svg(m_context, id);
    QString ret(str);
    dc_str_unref(str);
    return ret;
}

QString Context::getSecurejoinQr(uint32_t id)
{
    auto str = dc_get_securejoin_qr(m_context, id);
    QString ret(str);
    dc_str_unref(str);
    return ret;
}

uint32_t Context::joinSecurejoin(QString qrcode)
{
    auto qrdata = qrcode.toUtf8();
    return dc_join_securejoin(m_context, qrdata.constData());
}

} // namespace dc
